# -*- coding: utf-8 -*-
"""
Created on Wed Aug 03 16:44:41 2016

@author: Zhan
"""

from gurobipy import *
import numpy as np

class Input:
    def __init__(self):
        self.m = Model("rALP")
        
    def readIn(self,filename):        
        with open(filename) as file:
            next(file)
            self.n = int(next(file))
            
            for i in range(0,3):
                next(file)
            self.f = int(next(file))
            self.flight = []
            for i in range(0,self.f):
                self.flight.append([int(x) for x in next(file).split()])
                
            for i in range(0,3):
                next(file)
            self.i = int(next(file))
            self.iti = []
            for i in range(0,self.i):
                t = next(file).split()
                self.iti.append((int(t[0]),int(t[1]),int(t[2]),float(t[3])))
                
            for i in range(0,3):
                next(file)
            self.prob = []
            self.bdic = {}
            for i in range(0,self.n):
                t = next(file).split()
                r = []
                r.append(int(t[0]))
                j = 2
                while(j<len(t)):
                    r.append((int(t[j]),int(t[j+1]),int(t[j+2]),float(t[j+4])))
                    #bdic -- index (time,from,to,class) and return 
                    self.bdic[(i,int(t[j]),int(t[j+1]),int(t[j+2]))] = float(t[j+4])
                    j = j+6
                self.prob.append(r)
        
    def product(self):
        self.pdic = {}
        self.prdic = {}
        self.pval = []
        for i in range(0,self.i):
            t = self.iti[i]
            #pdic -- index (from,to,class) and return id of the product
            #its inverse prdic
            #self.pval[pdic] store the value
            self.pdic[(t[0],t[1],t[2])] = i
            self.prdic[i] = (t[0],t[1],t[2])
            self.pval.append(t[3])
    
    def leg(self):
        self.ldic = {}
        self.lrdic = {}
        self.leg = []
        for i in range(0,self.f):
            t = self.flight[i]
            #ldic -- index (from,to) and return id of the leg
            #its inverse lrdic
            #self.flight[i][2] store numbers of legs
            self.ldic[(t[0],t[1])] = i
            self.lrdic[i] = (t[0],t[1])

    def lam(self):
        self.lmda = []
        for t in range(0,self.n):
            lmda = {}
            
    
    def constrRefforIJ(self,a,b):
        b=b-1
        if self.dicAforI.has_key(a):
            self.dicAforI[a].append(b)
        else:
            self.dicAforI[a]=[b]
        if self.dicAforJ.has_key(b):
            self.dicAforJ[b].append(a)
        else:
            self.dicAforJ[b] = [a]
    
    def construct(self):
        #self.a = np.zeros((20,2000),dtype=np.int8)
        self.val = []
        self.id = 0
        self.listA = []
        self.ref = []
        #self.lmda = np.zeros((1000,200),dtype=np.float)
        self.dicAforI = {}
        self.dicAforJ = {}
        for j in range(0,self.i):
            t = self.prdic[j]
            if self.ldic.has_key((t[0],t[1])):
                self.id = self.id + 1
                self.ref.append(j)
                self.val.append(self.pval[j])
                #for p in range(0,self.n):
                    #self.lmda[p,self.id] = self.bdic[(p,t[0],t[1],t[2])]
                #self.a[self.ldic[(t[0],t[1])],self.id] = 1
                self.listA.append((self.ldic[(t[0],t[1])],self.id))
                self.constrRefforIJ(self.ldic[(t[0],t[1])],self.id)
            else:
                for k in range(0,self.f/2+1):
                    if k != t[0] and k != t[1] and self.ldic.has_key((t[0],k)) and self.ldic.has_key((k,t[1])):
                        self.id = self.id+1
                        self.ref.append(j)
                        #self.a[self.ldic[(t[0],k)],self.id] = 1
                        #self.a[self.ldic[(k,t[1])],self.id] = 1
                        self.val.append(self.pval[j])
                        #for p in range(0,self.n):
                            #self.lmda[p,self.id] = self.bdic[(p,t[0],t[1],t[2])]
                        self.listA.append((self.ldic[(t[0],k)],self.id))
                        self.listA.append((self.ldic[(k,t[1])],self.id))                
                        self.constrRefforIJ(self.ldic[(t[0],k)],self.id)                
                        self.constrRefforIJ(self.ldic[(k,t[1])],self.id)        

    def echoVal(self):
        print "id:%d"%self.id
        print "f:%d"%self.f
        print "i:%d"%self.i
        print self.prob
        print self.pval
        print self.flight
    
    def addVAR(self):
        self.q = {}
        self.y = {}
        self.z = {}
        for t in range(0,self.n):
            for i in range(0,self.id):
                self.q[t,i] = self.m.addVar(vtype=GRB.CONTINUOUS,name = 'q_%s_%s' % (t,i))
            for i in range(0,self.f):
                for k in range(0,self.flight[i][2]):
                    self.y[t,i,k] = self.m.addVar(vtype=GRB.CONTINUOUS,name = 'y_%s_%s_%s' % (t,i,k))
            for j in range(0,self.id):
                for i in range(0,self.f):
                    for k in range(0,self.flight[i][2]+1):
                        #print self.n*self.id*self.f*self.flight[i][2]
                        self.z[t,j,i,k] = self.m.addVar(vtype=GRB.CONTINUOUS,name = 'z_%s_%s_%s' % (t,j,i,k))
        self.m.update()
        
    def addOpt(self):
        return 


import numpy as np
from scipy.special import gamma

class SIM_Resolve_Second:
    def __init__(self, t=10):
        self.t = t        
        self.set_para()

    def set_para(self):
        self.i = 10
        self.j = 60
        
        #Fare
        self.v = np.zeros((self.j,1))
        for j in range(0,10):
            self.v[2*j] = 300
            self.v[2*j+1] = 80
        for j in range(10,22):
            self.v[2*j] = 500
            self.v[2*j+1] = 100
        for j in range(22,30):
            self.v[2*j] = 700
            self.v[2*j+1] = 200
            
        #Capacity
        self.c = np.zeros((self.i,1))
        for i in range(0,10):
            self.c[i] = 400
        self.c[4] = self.c[5] = 1000
        
        #matrix A 
        self.A = {}
        for j in range(0,10):
            self.A[2*j] = [j]
            self.A[2*j+1] = [j]
        self.pushTwoLeg(0,1,0,3,1,2,0)
        self.pushTwoLeg(0,5,0,4,5,1,1)
        self.pushTwoLeg(1,5,2,4,5,3,2)
        self.pushTwoLeg(2,4,6,5,4,7,3)
        self.pushTwoLeg(2,3,6,8,9,7,4)
        self.pushTwoLeg(3,4,9,5,4,8,5)
        self.pushThreeLeg(0,2,0,4,7,6,5,1,6)
        self.pushThreeLeg(0,3,0,4,8,9,5,1,7)
        self.pushThreeLeg(1,2,2,4,7,6,5,3,8)
        self.pushThreeLeg(1,3,2,4,8,9,5,3,9)
        
        totalLen = 1000.0
        div = int(totalLen / self.t)
        self.cons = np.zeros((self.t,2))
        for t in range(0,self.t):            
            for tt in range(t*div,(t+1)*div):
                self.cons[t,0] +=  0.25 * 1.0/totalLen * (float(tt)/totalLen) ** (6 - 1) * (1- float(tt)/totalLen) ** (2-1) * gamma(8)/gamma(2)/gamma(6)
                self.cons[t,1] +=  0.75 * 1.0/totalLen * (float(tt)/totalLen) ** (2 - 1) * (1- float(tt)/totalLen) ** (6-1) * gamma(8)/gamma(2)/gamma(6)
                
    
    def pushTwoLeg(self,s,t,a,b,c,d,id):
        self.A[20+4*id] = [a,b]
        self.A[20+4*id+1] = [a,b]
        self.A[20+4*id+2] = [c,d]
        self.A[20+4*id+3] = [c,d] 
    
    def pushThreeLeg(self,s,t,a,b,c,d,e,f,id):
        self.A[20+4*id] = [a,b,c]
        self.A[20+4*id+1] = [a,b,c] 
        self.A[20+4*id+2] = [d,e,f]
        self.A[20+4*id+3] = [d,e,f]

    def sim(self):    
        self.realDemand = np.zeros((self.t,self.j))
        for t in range(0,self.t):
            for j in range(0,10):
                g = np.random.gamma(60)
                self.realDemand[t,2*j]= np.random.poisson(g * self.cons[t][0])
                self.realDemand[t,2*j+1]= np.random.poisson(g * self.cons[t][1])
            for j in range(10,22):
                g = np.random.gamma(150)
                self.realDemand[t,2*j] = np.random.poisson(g * self.cons[t][0])
                self.realDemand[t,2*j+1] = np.random.poisson(g * self.cons[t][1])
            for j in range(22,30):
                g = np.random.gamma(100)
                self.realDemand[t,2*j] = np.random.poisson(g * self.cons[t][0])
                self.realDemand[t,2*j+1] = np.random.poisson(g * self.cons[t][1])
        return self.realDemand

    


from algs import *
import numpy as np
class tri_linear(algs):

    def sum_history(self,data):
        sum_history = np.zeros((self.T,self.J))
            
        for t in range(1,self.T):
            for j in range(self.J):
                sum_history[t,j] = sum_history[t-1,j] + data[t-1,j]
        return sum_history

    def train_history_bound(self):
        history_D = []
        for data in self.D:
            history_D.append(self.sum_history(data))

        self.history_low_bound = np.zeros((self.T,self.J))
        self.history_up_bound = np.zeros((self.T,self.J))
        for t in range(1,self.T):
            for j in range(self.J):
                stat = []
                for data in history_D:
                    stat.append(data[t,j])
                self.history_low_bound[t,j] = np.percentile(stat,0.5)
                self.history_up_bound[t,j] = np.percentile(stat,99.5)
                if self.history_low_bound[t,j] == self.history_up_bound[t,j]:
                    self.history_up_bound[t,j] += 1
        
        for j in range(self.J):
            self.history_up_bound[0,j] = 1.
            self.history_low_bound[0,j] = 0.

    def mesh(self):
        self.div_x_num = 7
        self.div_y_num = 10
        self.div_x_axis = {}
        self.div_y_axis = {}
        
        self.constant = self.script.constant
        self.xi = self.script.xi

        for t in range(self.T):
            for j in range(self.J):
                for x in range(self.div_x_num):
                    self.div_x_axis[t,j,x] = self.low_bound[t,j] + (self.up_bound[t,j]-self.low_bound[t,j])/float(self.div_x_num - 1) * x

                for y in range(self.div_y_num):
                    self.div_y_axis[t,j,y] = self.history_low_bound[t,j] + (self.history_up_bound[t,j]-self.history_low_bound[t,j])/float(self.div_y_num - 1) * y
                    
        for t in range(self.T):
            for j in range(self.J):
                for x in range(self.div_x_num):
                    for y in range(self.div_y_num):
                        self.xi[t,j,x,y] = self.script.add_var(0,self.div_x_axis[t,j,x])



    def piecewise(self):
        for t in range(self.T):
            for j in range(self.J):
                left = {}
                for x in range(self.div_x_num):
                    for y in range(self.div_y_num):
                        left[self.xi[t,j,x,y]] = 1
                right = {}
                right[self.constant] = 1
                self.script.add_lin_equ(left,right)

                for x in range(self.div_x_num):
                    for y in range(self.div_y_num):
                        self.script.add_lin_greater({self.xi[t,j,x,y]:1},{self.constant:0})

        for t in range(self.T):
            for j in range(self.J):
                left = {self.xi[t,j]:1}
                right = {}
                for x in range(self.div_x_num):
                    for y in range(self.div_y_num):
                        right[self.xi[t,j,x,y]] = self.div_x_axis[t,j,x]

                self.script.add_lin_equ(left,right)

                left = {}
                for r in range(t):
                    left[self.xi[r,j]] = 1
                right = {}
                for x in range(self.div_x_num):
                    for y in range(self.div_y_num):
                        right[self.xi[t,j,x,y]] = self.div_y_axis[t,j,y]

                self.script.add_lin_equ(left,right)

    def value(self,t,j,x,y,v_x,v_y):
        if y < self.div_y_num - 1 and v_y >= self.div_y_axis[t,j,y+1]:
            return 0
        if y > 0 and v_y <= self.div_y_axis[t,j,y-1]:
            return 0
        if x < self.div_x_num - 1 and v_x >= self.div_x_axis[t,j,x+1]:
            return 0
        if x > 0 and v_x <= self.div_x_axis[t,j,x-1] :
            return 0
        
        if x == self.div_x_num - 1 and v_x >= self.div_x_axis[t,j,self.div_x_num - 1] \
        or x == 0 and v_x <= self.div_x_axis[t,j,0]:
            if y == self.div_y_num - 1 and v_y >= self.div_y_axis[t,j,self.div_y_num -1] :
                return 1
            elif y < self.div_y_num - 1 and v_y >= self.div_y_axis[t,j,y+1]:
                return 0

            if y == 0 and v_y <= self.div_y_axis[t,j,0]:
                return 1
            elif y > 0 and v_y <= self.div_y_axis[t,j,y-1]:
                return 0

            if v_y > self.div_y_axis[t,j,y]:
                return (self.div_y_axis[t,j,y+1] - v_y) / (self.div_y_axis[t,j,y+1] - self.div_y_axis[t,j,y])
            else:
                return (v_y-self.div_y_axis[t,j,y-1]) / (self.div_y_axis[t,j,y] - self.div_y_axis[t,j,y-1])


        if y == self.div_y_num - 1 and v_y >= self.div_y_axis[t,j,self.div_y_num - 1] \
        or y == 0 and v_y <= self.div_y_axis[t,j,0]:
            if x == self.div_x_num - 1 and v_x >= self.div_x_axis[t,j,self.div_x_num - 1]:
                return 1
            elif x < self.div_x_num - 1 and v_x >= self.div_x_axis[t,j,x+1]:
                return 0

            if x == 0 and v_x <= self.div_x_axis[t,j,0]:
                return 1
            elif x > 0 and v_x <= self.div_x_axis[t,j,x-1]:
                return 0

            if v_x > self.div_x_axis[t,j,x]:
                return (self.div_x_axis[t,j,x+1] - v_x) / (self.div_x_axis[t,j,x+1] - self.div_x_axis[t,j,x])
            else:
                return (v_x-self.div_x_axis[t,j,x-1]) / (self.div_x_axis[t,j,x] - self.div_x_axis[t,j,x-1])
        
        def solve_linalg(x,y):
            '''
            print t,j
            print x
            print [self.div_x_axis[t,j,x[0]],self.div_x_axis[t,j,x[1]],self.div_x_axis[t,j,x[2]]]
            print y
            print [self.div_y_axis[t,j,y[0]],self.div_y_axis[t,j,y[1]],self.div_y_axis[t,j,y[2]]]
            
            latter = x[1]*y[2] - y[1]*x[2]
            return (v_x*y[1]+x[2]*v_y-y[2]*v_x-v_y*x[1] + latter)\
                    /(x[0]*y[1]+x[2]*y[0]-y[2]*x[0]-y[0]*x[1] + latter)
            '''
            A = np.zeros((3,3))
            b = np.zeros((3,1))
            A[0,0] = self.div_x_axis[t,j,x[0]]
            A[0,1] = self.div_x_axis[t,j,x[1]]
            A[0,2] = self.div_x_axis[t,j,x[2]]
            A[1,0] = self.div_y_axis[t,j,y[0]]
            A[1,1] = self.div_y_axis[t,j,y[1]]
            A[1,2] = self.div_y_axis[t,j,y[2]]
            A[2,:] = np.ones((1,3))
            b[0] = v_x
            b[1] = v_y
            b[2] = 1
            Lambda = np.linalg.solve(A,b)
            return Lambda[0]
            

        if v_y >= self.div_y_axis[t,j,y] and v_x >= self.div_x_axis[t,j,x]:
            p = np.array([v_x - self.div_x_axis[t,j,x], v_y - self.div_y_axis[t,j,y+1]])
            q = np.array([self.div_x_axis[t,j,x+1] - self.div_x_axis[t,j,x], self.div_y_axis[t,j,y] - self.div_y_axis[t,j,y+1]])
            if np.cross(p,q) <= 0:
                return 0
            else:
                return solve_linalg([x,x,x+1],[y,y+1,y])

        if v_y >= self.div_y_axis[t,j,y] and v_x <= self.div_x_axis[t,j,x]:
            '''
            print x,y
            print v_x,v_y
            print self.div_x_axis[t,j,x], self.div_y_axis[t,j,y]
            '''
            p = np.array([v_x - self.div_x_axis[t,j,x], v_y - self.div_y_axis[t,j,y]])
            q = np.array([self.div_x_axis[t,j,x-1] - self.div_x_axis[t,j,x], self.div_y_axis[t,j,y+1] - self.div_y_axis[t,j,y]])
            if np.cross(p,q) >= 0:
                return solve_linalg([x,x,x-1],[y,y+1,y+1])
            else:
                return solve_linalg([x,x-1,x-1],[y,y+1,y])

        if v_y <= self.div_y_axis[t,j,y] and v_x >= self.div_x_axis[t,j,x]:
            p = np.array([v_x - self.div_x_axis[t,j,x], v_y - self.div_y_axis[t,j,y]])
            q = np.array([self.div_x_axis[t,j,x+1] - self.div_x_axis[t,j,x], self.div_y_axis[t,j,y-1] - self.div_y_axis[t,j,y]])
            if np.cross(p,q) >= 0:
                return solve_linalg([x,x,x+1],[y,y-1,y-1])
            else:
                return solve_linalg([x,x+1,x+1],[y,y,y-1])

        if v_y <= self.div_y_axis[t,j,y] and v_x <= self.div_x_axis[t,j,x]:
            p = np.array([v_x - self.div_x_axis[t,j,x], v_y - self.div_y_axis[t,j,y-1]])
            q = np.array([self.div_x_axis[t,j,x-1] - self.div_x_axis[t,j,x], self.div_y_axis[t,j,y] - self.div_y_axis[t,j,y-1]])
            if np.cross(p,q) > 0:
                return solve_linalg([x,x,x-1],[y,y-1,y])
            else:
                return 0


    def lift_value(self,data):
        sum_data = self.sum_history(data)
        value = np.zeros((self.script.index,1))
        value[self.constant] = 1
        for t in range(self.T):
            for j in range(self.J):
                value[self.xi[t,j]] = data[t,j]
                for x in range(self.div_x_num):
                    for y in range(self.div_y_num):
                        value[self.xi[t,j,x,y]] = self.value(t,j,x,y,data[t,j],sum_data[t,j])
                        
        return value

    def relation(self):
        self.P = {}
        for t in range(self.T):
            for j in range(self.J):
                self.P[t,j] = []
                for x in range(self.div_x_num):
                    for y in range(self.div_y_num):
                        self.P[t,j].append(self.xi[t,j,x,y])
    
    def strategy(self,t,history,X):
        '''
        print self.J
        print history
        '''
        sum_history = self.sum_history(history)
        product = np.zeros((self.script.J,1))
        for j in range(self.script.J):
            if sum_history[t,j] <= self.history_low_bound[t,j]:
                product[j] = X[t,j,self.xi[t,j,self.div_x_num-1,0]].X
            elif sum_history[t,j] >= self.history_up_bound[t,j]:
                product[j] = X[t,j,self.xi[t,j,self.div_x_num-1,self.div_y_num-1]].X
            else:
                for k in range(1,self.div_y_num):
                    if self.div_y_axis[t,j,k] > sum_history[t,j]:
                        break
                    
                coef = (sum_history[t,j] - self.div_y_axis[t,j,k-1]) / float(self.div_y_axis[t,j,k]-self.div_y_axis[t,j,k-1])
                product[j] = (1-coef) * X[t,j,self.xi[t,j,self.div_x_num-1,k-1]].X + coef * X[t,j,self.xi[t,j,self.div_x_num-1,k]].X
        return product

    def update(self):
        self.train_history_bound()
        self.mesh()
        self.piecewise()
        self.relation()


# -*- coding: utf-8 -*-
"""
Created on Sun Aug 07 13:12:01 2016

@author: Zhan
"""

from gurobipy import *
import numpy as np

class rALP:
    def __init__(self,relrALP):
        self.m = Model("Decision Rule Approch")
        self.i = relrALP.f
        self.j = relrALP.i
        self.t = relrALP.n
        
        
        self.t = 200#customized periods
        
        
        #Sparse P
        #self.P = np.zeros((20000,20000),dtype=np.float)
        self.A = np.zeros((self.i,self.j),dtype=np.float)
        self.xi = np.zeros((self.t*self.j+1,1),dtype=np.float)
        self.c = np.zeros((self.i,1),dtype=np.float)
        self.h = np.zeros((2*(self.t*self.j+1),1),dtype=np.float)
        #Sparse W
        #self.W = np.zeros((2*(self.t*self.j+1),self.t*self.j+1),dtype=np.float)
        self.v = np.zeros((self.j,1),dtype=np.float)
        self.rALP = relrALP
        self.refI = relrALP.dicAforI
        self.refJ = relrALP.dicAforJ
        
    def construct(self):
        #construct A
        for item in self.rALP.listA:
            #Hint Item[1] is from 1 to ...
            #print item
            self.A[item[0],item[1]-1] = 1
        #construct c
        for item in range(0,self.i):
            self.c[item] = self.rALP.flight[item][2]
            #self.c[item] = 10
        #construct v
        for j in range(0,self.j):
            self.v[j] = self.rALP.pval[j]
        #construct Exi
        self.xi[0] = 1
        for t in range(0,self.t):
            for j in range(0,self.j):
                index = self.rALP.prdic[j]
                self.xi[1+t*self.j+j] = self.rALP.bdic[(t,index[0],index[1],index[2])]
                
    def echoInput(self):
        print "i:%d"%self.i
        print "j:%d"%self.j
        print "t:%d"%self.t
        print "legs:"
        print self.c
        print "value:"
        print self.v
        print "h:"
        print self.h
        print "A:"
        print self.A
        print "xi:"
        print self.xi
    
    def addVar(self):
        #add y
        self.y = {}
        for t in range(0,self.t):
            for i in range(0,self.i):
                for k in range(0,self.c[i]):
                    self.y[t,i,k] = self.m.addVar(lb=-GRB.INFINITY, name = 'y %d %d %d' %(t,i,k) )
        self.z = {}
        for t in range(0,self.t):
            for j in range(0,self.j):
                for i in range(0,self.i):
                    for k in range(0,int(self.c[i])):
                        self.z[t,j,i,k] = self.m.addVar(lb=-GRB.INFINITY, name = 'z %d %d %d %d' %(t,j,i,k) )
                    self.z[t,j,i,int(self.c[i,0])] = self.m.addVar(ub=0,lb=0,name = 'z %d %d %d %d' %(t,j,i,self.c[i,0]) )
        self.q = {}
        for t in range(0,self.t):
            for j in range(0,self.j):
                self.q[t,j] = self.m.addVar(lb=-GRB.INFINITY, name = 'q %d %d' %(t,j) )
        self.m.update()
    
    def addOpt(self):
        obj  = LinExpr()
        for t in range(0,self.t):
            for j in range(0,self.j):
                obj += self.xi[1+t*self.j+j,0] * self.v[j,0] * self.q[t,j]
        self.m.setObjective(obj,GRB.MAXIMIZE)
    
    def addConstr(self):
        #y
        for i in range(0,self.i):
            for k in range(0,self.c[i]):
                self.m.addConstr(self.y[0,i,k],GRB.EQUAL,1,name = 'Constr y %d %d %d' % (0,i,k) )
        for t in range(1,self.t):
            for i in range(0,self.i):
                for k in range(0,self.c[i]):
                    rhs = LinExpr()
                    for j in self.refI[i]:
                        rhs += self.xi[1+(t-1)*self.j+j,0] * (self.z[t-1,j,i,k] - self.z[t-1,j,i,k+1])
                    self.m.addConstr(self.y[t-1,i,k]-self.y[t,i,k],GRB.EQUAL,rhs,name = 'Constr y %d %d %d' %(t,i,k))
        #q
        for t in range(0,self.t):
            for j in range(0,self.j):
                for i in self.refJ[j]:
                    self.m.addConstr(self.q[t,j],GRB.EQUAL,self.z[t,j,i,1],name = 'Constr q %d %d %d' %(t,j,i))
        #z
        for t in range(0,self.t):
            for j in range(0,self.j):
                for i in self.refJ[j]:
                    for k in range(0,self.c[i]):
                        self.m.addConstr(self.z[t,j,i,k+1],GRB.LESS_EQUAL,self.z[t,j,i,k],name = 'Constr z %d %d %d %d'%(t,j,i,k+1))
                        self.m.addConstr(self.z[t,j,i,k],GRB.LESS_EQUAL,self.y[t,i,k],name = 'Constr z<=y %d %d %d %d'%(t,j,i,k))
                        
    def solve(self):
        self.m.optimize()
    
    def echoOpt(self):
        print self.m.getObjective().getValue()
    
    def writeMPS(self):
        self.m.write("out.mps")
    
#    def echoSolution(self):



import numpy as np
class algs(object):

    def get_train_bound(self):
        return self.low_bound, self.up_bound

    def train_bound(self):
        self.low_bound = np.zeros((self.T,self.J))
        self.up_bound = np.zeros((self.T,self.J))
        for t in range(self.T):
            for j in range(self.J):
                stat = []
                for data in self.D:
                    stat.append(data[t,j])
                self.low_bound[t,j] = np.percentile(stat,0.5)
                self.up_bound[t,j] = np.percentile(stat,99.5)
                
                self.low_bound[t,j] = max(self.low_bound[t,j]-2,0)
                if self.low_bound[t,j] == self.up_bound[t,j]:
                    self.up_bound[t,j] += 1
    
    def set_D(self,D):
        self.D = D
        self.T = np.shape(self.D[0])[0]
        self.J = np.shape(self.D[0])[1]
        self.train_bound()

    def set_script(self,script):
        self.script = script

    def get_relation(self):
        return self.P




# -*- coding: utf-8 -*-
"""
Created on Thu Feb 23 19:19:54 2017

@author: dy403
"""

import numpy as np
from gurobipy import *

class LDR:
    def __init__(self,A,E,W,h,P,V,T,J,C,xi,lb,ub):
        self.A = A
        self.E = E
        self.W = W
        self.h = h
        self.P = P
        self.V = V
        self.T = T
        self.J = J
        self.C = C
        self.L = len(self.C)
        self.xi = xi
        self.lb = lb
        self.ub = ub

    def build_model(self):
        self.model = Model("Decision Rule Approch")
        self.X = {}
        for t in range(self.T):
            for j in range(self.J):
                for k in self.P[t,j]:
                    self.X[t,j,k] = self.model.addVar(lb = self.lb[k], ub = self.ub[k])

        obj = LinExpr()
        for t in range(self.T):
            for j in range(self.J):
                for k in self.P[t,j]:
                    obj += self.X[t,j,k] * self.V[j] * self.E[k]

        self.model.setObjective(obj,GRB.MAXIMIZE)


        print "Constriant:Used Resource <= Total Resource"
        self.invA = {}
        for l in range(self.L):
            self.invA[l] = []

        for (a,b) in self.A.items():
            for r in b:
                self.invA[r].append(a)

        for l in range(self.L):
            left = {}
            for j in self.invA[l]:
                for t in range(self.T):
                    for k in self.P[t,j]:
                        if k in left:
                            left[k] += self.X[t,j,k]
                        else:
                            left[k] = self.X[t,j,k]

            right = {0:LinExpr()}
            right[0] += self.C[l] * 2
            for j in self.invA[l]:
                for t in range(self.T):
                    for k in self.P[t,j]:
                        right[0] = right[0] - self.X[t,j,k] * self.E[k]
            self.add_less(left,right)

        print "Update Model"
        self.model.update()

    def add_less(self,left,right):
        self.add_greater(right,left)

    def add_greater(self,left,right):
        self.row_W = np.shape(self.W)[0]
        self.col_W = np.shape(self.W)[1]
        Lambda = {}
        for i in range(self.row_W):
            Lambda[i] = self.model.addVar(lb = 0, ub = GRB.INFINITY)

        for (a,b) in right.items():
            if a in left:
                left[a] -= b
            else:
                left[a] = -b

        for i in left.keys():
            temp = LinExpr()
            col_index = self.W[:,i].indices
            col_data = self.W[:,i].data
            for k in range(len(col_data)):
                j = col_index[k]
                temp += Lambda[j] * col_data[k]

            self.model.addConstr(temp == left[i])

        I = set(range(self.col_W))
        DIFF = set(left.keys())
        I = I.difference(DIFF)

        for i in I:
            temp = LinExpr()
            col_index = self.W[:,i].indices
            col_data = self.W[:,i].data
            for k in range(len(col_data)):
                j = col_index[k]
                temp += Lambda[j] * col_data[k]


            self.model.addConstr(temp == 0)

        temp = LinExpr()
        for i in range(self.row_W):
            temp += Lambda[i] * self.h[i]

        self.model.addConstr(temp >= 0)

    def solve_model(self):
        print "Optimize Model"
        self.model.optimize()


from tri_linear import *
import numpy as np
from gurobipy import *
from scipy.optimize import minimize
from inter import *
class opt_tri_linear(tri_linear):

#Method 1: Gradient Descent

#Method 2: Alternating Iterative

#Now I Use Gradient Descent

    def optimize_x(self,theta):             
        
        self.invA = {}
        for l in range(self.script.L):
            self.invA[l] = []

        for (a,b) in self.script.A.items():
            for i in b:
                self.invA[i].append(a)
                
        #print theta
        
        self.model = Model("Optimize X given theta")
        self.X = {}
        for t in range(self.T):
            for j in range(self.J):
                for x in range(self.div_x_num):
                    for y in range(self.div_y_num):
                        self.X[t,j,x,y] = self.model.addVar(lb = 0, ub = self.div_x_axis[t,j,x])

        obj = LinExpr()
        for data in self.train_data:
            sum_data = self.sum_history(data)
            for t in range(self.T):
                for j in range(self.J):
                    x_val = data[t,j]
                    y_val = 0
                    for k in range(self.J):
                        y_val += theta[j,k] * sum_data[t,k]
                        
                    for x in range(self.div_x_num):
                        for y in range(self.div_y_num):
                            obj += self.X[t,j,x,y] * self.value(t,j,x,y,x_val,y_val) * self.script.V[j]
                            #print test_var, self.value(t,j,x,y,x_val,y_val) 
        obj = obj / float(len(self.train_data))
        self.model.setObjective(obj,GRB.MAXIMIZE)

        '''
            for t in range(self.T):
                for j in range(self.J):
                    constriant = LinExpr()
                    x_val = data[t,j]
                    y_val = 0
                    for k in range(self.J):
                        y_val += theta[j,k] * sum_data[t,k]

                    for x in range(self.div_x_num):
                        for y in range(self.div_y_num):
                            constriant += self.X[t,j,x,y] * self.value(t,j,x,y,x_val,y_val)
                    model.addConstr(constriant >= 0)

            for t in range(self.T):
                for j in range(self.J):
                    constriant = LinExpr()
                    x_val = data[t,j]
                    y_val = 0
                    for k in range(self.J):
                        y_val += theta[j,k] * sum_data[t,j]

                    for x in range(self.div_x_num):
                        for y in range(self.div_y_num):
                            constriant += self.X[t,j,x,y] * self.value(t,j,x,y,x_val,y_val)
                    model.addConstr(constriant <= data[t,j])

                    '''
        for data in self.train_data:
            sum_data = self.sum_history(data)
            for l in range(self.script.L):
                constriant = LinExpr()
                for j in self.invA[l]:
                    for t in range(self.T):
                        x_val = data[t,j]
                        y_val = 0
                        for k in range(self.J):
                            y_val += theta[j,k] * sum_data[t,k]
                        for x in range(self.div_x_num):
                            for y in range(self.div_y_num):
                                constriant += self.X[t,j,x,y] * self.value(t,j,x,y,x_val,y_val)
                                
                        #print t,j,x,y,x_val,y_val
                        #print y_val-self.div_y_axis[t,j,0],self.div_y_axis[t,j,self.div_y_num-1] - y_val
                self.model.addConstr(constriant <= self.script.C[l])

        self.model.update()
        self.model.optimize()
        '''
        for t in range(self.T):
            for j in range(self.J):
                for x in range(self.div_x_num):
                    for y in range(self.div_y_num):
        '''
        
        return self.model.getObjective().getValue()

    def set_train_data(self,train_data):
        self.train_data = train_data

    def piecewise(self):
        self.constant = self.script.constant
        self.xi = self.script.xi
        self.sum_xi = {}
        for t in range(0,self.T):
            for j in range(self.J):
                self.sum_xi[t,j] = self.script.add_var(0,0)

        for t in range(1,self.T):
            for j in range(self.J):
                left = {self.sum_xi[t,j]:1}
                right = {}
                for u in range(t):
                    right[self.xi[u,j]] = 1
                self.script.add_lin_equ(left,right)

        for t in range(self.T):
            for j in range(self.J):
                self.y_var = self.script.add_var(0,0)

        for t in range(self.T):
            for j in range(self.J):
                left = {self.y_var:1}
                right = {}
                for k in range(self.J):
                    right[self.sum_xi[t,k]] = self.theta[j,k]
                self.script.add_lin_equ(left,right)

        for t in range(self.T):
            for j in range(self.J):
                left = {}
                for x in range(self.div_x_num):
                    for y in range(self.div_y_num):
                        left[self.xi[t,j,x,y]] = 1
                right = {}
                right[self.constant] = 1
                self.script.add_lin_equ(left,right)

                for x in range(self.div_x_num):
                    for y in range(self.div_y_num):
                        self.script.add_lin_greater({self.xi[t,j,x,y]:1},{self.constant:0})

        for t in range(self.T):
            for j in range(self.J):
                left = {self.xi[t,j]:1}
                right = {}
                for x in range(self.div_x_num):
                    for y in range(self.div_y_num):
                        right[self.xi[t,j,x,y]] = self.div_x_axis[t,j,x]

                self.script.add_lin_equ(left,right)

                left = {self.y_var[t,j]:1}
                right = {}
                for x in range(self.div_x_num):
                    for y in range(self.div_y_num):
                        right[self.xi[t,j,x,y]] = self.div_y_axis[t,j,y]

                self.script.add_lin_equ(left,right)
    
    def lift_value(self,data):
        sum_data = self.sum_history(data)
        value = np.zeros((self.script.index,1))
        value[self.constant] = 1
        for t in range(self.T):
            for j in range(self.J):
                value[self.xi[t,j]] = data[t,j]
                t_sum = 0
                if t >= 1:
                    for k in range(self.J):
                        t_sum = sum_data[t,k] * self.theta[j,k]
                for x in range(self.div_x_num):
                    for y in range(self.div_y_num):
                        value[self.xi[t,j,x,y]] = self.value(t,j,x,y,data[t,j],t_sum)
                        
        return value
    
    def set_inter(self,inter):
        self.inter = inter
    
    def once(self,sample):
        C = self.script.C.copy()
        V = self.script.V
        val = 0
        his = self.sum_history(sample)
        for t in range(self.script.T):
            product = self.strategy(t,sample, his)
            
            #product = product.reshape(self.script.J)
            
            for j in range(self.script.J):
                product[j] = self.inter(t,j,product[j])
                product[j] = min(product[j], sample[t,j])
                
                for l in self.script.A[j]:
                    product[j] = min(product[j],C[l])

                val += product[j] * V[j]
                for l in self.script.A[j]:
                    C[l] -= product[j]

        return val
            
    
    def strategy(self,t,data,sum_data):
            obj = np.zeros(self.J)
            for j in range(self.J):
                x_val = data[t,j]
                y_val = 0
                for k in range(self.J):
                    y_val += self.theta[j,k] * sum_data[t,k]

                for x in range(self.div_x_num):
                    for y in range(self.div_y_num):
                        obj[j] += self.X[t,j,x,y].X * self.value(t,j,x,y,x_val,y_val) 
                        #print test_var, self.value(t,j,x,y,x_val,y_val) 
            return obj
            '''
            sum_data = self.sum_history(history)
            product = np.zeros((self.script.J,1))
            for j in range(self.script.J):
                t_sum = 0
                if t >= 1:
                    for k in range(self.J):
                        t_sum = sum_data[t,k] * self.theta[j,k]
                        
                if t_sum <= self.history_low_bound[t,j]:
                    product[j] = X[t,j,self.div_x_num-1,0].X
                elif t_sum >= self.history_up_bound[t,j]:
                    product[j] = X[t,j,self.div_x_num-1,self.div_y_num-1].X
                else:
                    for k in range(1,self.div_y_num):
                        if self.div_y_axis[t,j,k] > t_sum:
                            break
                        
                    coef = (t_sum - self.div_y_axis[t,j,k-1]) / float(self.div_y_axis[t,j,k]-self.div_y_axis[t,j,k-1])
                    product[j] = (1-coef) * X[t,j,self.div_x_num-1,k-1].X + coef * X[t,j,self.div_x_num-1,k].X
            return product
            '''

    def update(self):
        self.train_history_bound()
        self.mesh()
        self.theta = np.identity(self.J)
        self.optimize_x(self.theta)



from SIM import *
from SIM_Resolve_First import *
from piecewise_linear import *
from tri_linear import *
from alter_solve import *
from script import *


case = SIM_Resolve_First()
#algs = piecewise_linear()
#algs = tri_linear()
algs = opt_tri_linear()
sim = SIM(case,algs)

data = []
for i in range(10):
    data.append(sim.simulator.sim())
algs.set_train_data(data)

sim.generate_data()
sim.script = script(sim.simulator.A,sim.D,sim.simulator.v,sim.simulator.c,sim.algs)
sim.algs.update()

algs.once(sim.sim())


# -*- coding: utf-8 -*-
"""
Created on Wed Feb 22 16:33:30 2017

@author: dy403
"""

import numpy as np
import scipy as sp

class MSSP:
    
    def __init__(self):
        self.demandPre = []
        self.pricePre = []
        self.APre = []
        self.cPre = []
        self.bPre = []
        self.WPre = []
        self.demand = []
        self.price = []
        self.A = []
        self.c = []
        self.b = []
        self.W = []
        self.h = []
        
    #Number of Stage
    def setStage(self,t):
        self.t = t
    
    #Number of Legs
    
    def setLeg(self,l):
        self.l = l
    
    #Number of Classes
    def setClass(self,cl):
        self.cl = cl
    
    #Add Demand
    def addDemand(self,d):
        self.demand.append(d)
    
    def setDemand(self,d):
        self.demand = d
    
    #Add usage to construct A, only non-zero item needed
    def addA(self,i,j,t):
        self.APre.append((i,j,t))
    
    #Add C
    def setC(self,i,c):
        self.cPre.append((i,c))
    
    #Add b
    def setb(self,t,b):
        self.bPre.append((t,b))
    
    #Add W, only non-zero item needed
    def setW(self,i,j,value):
        self.WPre.append((i,j,value))
    
    def seth(self,i,h):
        self.h.append((i,h))
    
    def constructA(self):
        data = []
        row = []
        col = []
        self.cltoleg = np.empty(self.cl,list)
        for i in range(self.cl):
            self.cltoleg[i] = []
        for item in self.APre:
            row.append(item[0])
            col.append(item[1])
            data.append(item[2])
            self.cltoleg[item[1]].append(item[0])
        self.A = sp.sparse.coo_matrix((data,(row,col)))
        #Use non-sparse structure temporarily
        self.A = self.A.todense()
        
    def constructW(self):
        data = []
        row = []
        col = []
        for item in self.WPre:
            row.append(item[0])
            col.append(item[1])
            data.append(item[2])
        self.W = sp.sparse.coo_matrix((data,(row,col)))
        #Use non-sparse structure temporarily
        self.W = self.W.todense()
        
    def constructh(self):
        tmp = np.zeros(np.shape(self.W)[0])
        for item in self.h:
            tmp[item[0]] = item[1]
        self.h = np.arry(tmp)
    
    def constructb(self):
        self.b = np.zeros(self.l)
        for item in self.bPre:
            t = item[0]
            real = item[1]
            self.b[t] = real
    
    def constructc(self):
        self.c = np.zeros(self.cl)
        for item in self.cPre:
            t = item[0]
            real = item[1]
            self.c[t] = real
            
    #Construct coefficient
    def update(self):
        self.constructA()
        self.constructW()
        self.constructh()
        self.constructb()
        self.constructc()


from processer import *
from script import *
from inter import *
class SIM(object):
    def __init__(self,simulator,algs):
        self.simulator = simulator
        self.algs = algs

    def generate_data(self):
        train_num = 1000
        self.D = []
        for i in range(train_num):
            self.D.append(self.simulator.sim())

    def update(self):
        self.generate_data()
        self.script = script(self.simulator.A,self.D,self.simulator.v,self.simulator.c,self.algs)
        self.algs.update()
        self.script.update()
        self.processer = processer(self.script)
        self.processer.set_inter(inter_ceil)
        self.processer.update()

    def train_mab(self):
        train_data = []
        train_mab_num = 100
        for i in range(train_mab_num):
            train_data.append(self.simulator.sim())
        self.processer.train(train_data)
        self.processer.set_inter(self.processer.mab_inter.inter)

    def once(self):
        sample = self.simulator.sim()
        return self.processer.test(sample)
    
    def avg(self,times):
        ans = 0.
        for i in range(times):
            ans += self.once()
        
        return ans / float(times)


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Sun Feb 26 18:57:03 2017

@author: root
"""
import sys
sys.path.append('..\MSSP')
import MSSP

class NETWORK:
    def __init__(self):
        self.resource = {}
        self.profit = {}
        self.usage = {}
        self.demand = []
        
    def addRes(self,num,res):
        self.resource[num] = res
    
    def addProfit(self,num,pro):
        self.profit[num] = pro
    
    def addUsage(self,num,usage):
        self.usage[num] = usage

    def setStage(self,stage):
        self.stage = stage
    
    def addDemand(self,demand):
        self.demand += demand
    
    def update(self):
        self.mssp = MSSP()
        self.mssp.setStage(self.stage)
        self.mssp.setLeg(len(self.resource.keys()))
        self.mssp.setClass(len(self.profit.keys()))
        for item in self.usage.items():
            self.mssp.addA(item[0],item[1],-1)
        for item in self.usage.items():
            self.mssp.setPrice(item[0],item[1])
        self.mssp.setDemand(self.demand)
        """
        self.mssp.setC(0,1)
        for i in range(self.stage*self.mssp.cl):
            self.mssp.setC(i+1,0)
        for i in range(self.cl):
            self.mssp.setB(0,i,-1)
        for i in range(1,self.stage+1):
            for j in range(self.mssp.cl):
                self.mssp.setB(i,j,0)
        """
        self.mssp.update()
            
    


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon May 15 19:19:11 2017

@author: lin
"""

from numpy import ceil,floor

def inter_ceil(t,j,v):
	return floor(v)+1

def inter_floor(t,j,v):
    return floor(v)


# -*- coding: utf-8 -*-
"""
Created on Thu Feb 23 17:25:20 2017

@author: dy403
"""


import numpy as np
from LDR import *

class DR:
    def __init__(self,script):
        self.script = script
        self.ldr = LDR(A = script.A, E = script.E, W = script.W, h = script.h, P = script.P, V = script.V, T = script.T, J = script.J, C = script.C, xi = script.xi, lb = script.lb, ub = script.ub)

    def update(self):
        self.ldr.build_model()
        self.ldr.solve_model()
        self.X = self.ldr.X

    def strategy(self,t,history):
        tran_his = np.vstack((history,np.zeros(((self.script.T-t),self.script.J))))
        return self.script.algs.strategy(t,tran_his,self.X)
        #tran_his = self.script.algs.lift_value(tran_his)
        #for j in range(self.script.J):
         #   for k in self.script.P[t,j]:
          #      product[j] += self.X[t,j,k].X * tran_his[k]



import numpy as np
from scipy.special import gamma

class SIM_Resolve_First(object):
    def __init__(self, t=10):
        self.t = t        
        self.set_para()

    def set_para(self):   
        self.i = 10
        self.j = 60
        
        #Fare
        self.v = np.zeros((self.j,1))
        for j in range(0,10):
            self.v[2*j] = 300
            self.v[2*j+1] = 80
        for j in range(10,30):
            self.v[2*j] = 500
            self.v[2*j+1] = 100
            
        #Capacity
        self.c = np.zeros((self.i,1))
        for i in range(0,10):
            self.c[i] = 400
        
        #matrix A I * J
        self.A = {}
        for j in range(0,10):
            self.A[2*j] = [j]
            self.A[2*j+1] = [j]
            
        for a in range(0,5):
            for b in range(0,5):
                if a == b:
                    continue
                j = j + 1

                self.A[2*j] = [2*a,2*b+1]
                self.A[2*j+1] = [2*a,2*b+1]
            
        totalLen = 1000.0
        div = int(totalLen / self.t)
        self.cons = np.zeros((self.t,2))
        for t in range(0,self.t):            
            for tt in range(t*div,(t+1)*div):
                self.cons[t,0] +=  0.25 * 1.0/totalLen * (float(tt)/totalLen) ** (6 - 1) * (1- float(tt)/totalLen) ** (2-1) * gamma(8)/gamma(2)/gamma(6)
                self.cons[t,1] +=  0.75 * 1.0/totalLen * (float(tt)/totalLen) ** (2 - 1) * (1- float(tt)/totalLen) ** (6-1) * gamma(8)/gamma(2)/gamma(6)

    def sim(self):        
        self.realDemand = np.zeros((self.t,self.j))
        for t in range(0,self.t):
            for j in range(0,10):
                g = np.random.gamma(40)
                self.realDemand[t,2*j] = np.random.poisson(g * self.cons[t][0])
                self.realDemand[t,2*j+1] = np.random.poisson(g * self.cons[t][1])
            for j in range(10,30):
                g = np.random.gamma(100)
                self.realDemand[t,2*j] = np.random.poisson(g * self.cons[t][0])
                self.realDemand[t,2*j+1] = np.random.poisson(g * self.cons[t][1])
        return self.realDemand


import numpy as np
import scipy as sp
class script(object):
    def __init__(self,A,D,V,C,algs):
        self.A = A
        self.D = D
        self.V = V
        self.C = C
        self.L = len(self.C)
        self.algs = algs
        self.index = 0
        self.lin_inequ_W = []
        self.lin_equ_W = []
        self.algs.set_D(self.D)
        self.algs.set_script(self)
        self.init_var()
        

    def init_var(self):
        self.train_num = len(self.D)
        self.T = np.shape(self.D[0])[0]
        self.J = np.shape(self.D[0])[1]

        self.low_bound, self.up_bound = self.algs.get_train_bound()

        self.xi = {}
        self.lb = {}
        self.ub = {}
        INF = 9999999
        self.constant = self.add_var(-INF,INF)
        for t in range(self.T):
            for j in range(self.J):
                self.xi[t,j] = self.add_var(0,0)
                self.add_lin_greater({self.xi[t,j]:1},{self.constant:self.low_bound[t,j]})
                self.add_lin_less({self.xi[t,j]:1},{self.constant:self.up_bound[t,j]})

    def add_var(self,lb,ub):
        self.lb[self.index] = lb
        self.ub[self.index] = ub
        self.index += 1
        return self.index - 1

    def add_lin_greater(self,left,right):
        self.add_lin_inequ(left,right)

    def add_lin_less(self,left,right):
        self.add_lin_inequ(right,left)

    def add_lin_inequ(self,left,right):
        self.lin_inequ_W.append((left,right))

    def add_lin_equ(self,left,right):
        self.lin_equ_W.append((left,right))

    def update(self):
        self.row_W = 2 + len(self.lin_inequ_W) + 2 * len(self.lin_equ_W)
        self.col_W = self.index
        self.W = sp.sparse.dok_matrix((self.row_W,self.col_W))
        #self.W = {}
        self.h = np.zeros((self.row_W,1))
        self.W[0,self.constant] = 1
        self.W[1,self.constant] = -1
        self.h[0] = 1
        self.h[1] = -1
        cur_row = 2
        for (left,right) in self.lin_inequ_W:
            for (a,b) in left.items():
                    self.W[cur_row,a] += b
                          
            for (a,b) in right.items():
                   self.W[cur_row,a] += -b
            cur_row += 1

        for (left,right) in self.lin_equ_W:
            for (a,b) in left.items():
                    self.W[cur_row,a] += b
                          
            for (a,b) in right.items():
                   self.W[cur_row,a] += -b
            cur_row += 1

            for (a,b) in left.items():
                    self.W[cur_row,a] += -b
                          
            for (a,b) in right.items():
                   self.W[cur_row,a] += b
            cur_row += 1
        '''
        for (left,right) in self.lin_inequ_W:
            for (a,b) in left.items():
                if (cur_row,a) in self.W:
                    self.W[cur_row,a] += b
                else:
                    self.W[cur_row,a] = b
                          
            for (a,b) in right.items():
                if (cur_row,a) in self.W:
                   self.W[cur_row,a] += -b
                else:
                    self.W[cur_row,a] = -b
            cur_row += 1

        for (left,right) in self.lin_equ_W:
            for (a,b) in left.items():
                if (cur_row,a) in self.W:
                    self.W[cur_row,a] += b
                else:
                    self.W[cur_row,a] = b
                          
            for (a,b) in right.items():
                if (cur_row,a) in self.W:
                   self.W[cur_row,a] += -b
                else:
                    self.W[cur_row,a] = -b
            cur_row += 1

            for (a,b) in left.items():
                if (cur_row,a) in self.W:
                    self.W[cur_row,a] += -b
                else:
                    self.W[cur_row,a] = -b
                          
            for (a,b) in right.items():
                if (cur_row,a) in self.W:
                   self.W[cur_row,a] += b
                else:
                    self.W[cur_row,a] = b
            cur_row += 1
        '''

        self.E = np.zeros((self.index,1))
        for data in self.D:
            self.E += self.algs.lift_value(data)
        self.E /= float(len(self.D))

        self.P = self.algs.get_relation()
        '''
        row = []
        col = []
        data = []
        for (a,b) in self.W.items():
            row.append(a[0])
            col.append(a[1])
            data.append(b)
            
        self.W = sp.sparse.csc_matrix((data,(row,col)))
        '''
        self.W = self.W.tocsc()
        
        #print "Sparse Ratio:", self.W.getnnz()/float(self.row_W*self.col_W)


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Wed May 17 19:27:37 2017

@author: lin
"""


from SIM import *
from SIM_Resolve_First import *
from piecewise_linear import *
from tri_linear import *
from opt_tri_linear import *
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D

case = SIM_Resolve_First()
#algs = piecewise_linear()
#algs = tri_linear()
algs = opt_tri_linear()
sim = SIM(case,algs)

data = []
for i in range(30):
    data.append(sim.simulator.sim())
algs.set_train_data(data)

sim.generate_data()
sim.script = script(sim.simulator.A,sim.D,sim.simulator.v,sim.simulator.c,sim.algs)
algs.train_history_bound()
algs.mesh()

t = 3
j = 3
x = 0
y = 4
xx = np.linspace(algs.low_bound[t,j],algs.up_bound[t,j],20)
yy = np.linspace(algs.history_low_bound[t,j], algs.history_up_bound[t,j],20)
X, Y = np.meshgrid(xx,yy)
mem = np.zeros((20,20))
for a in range(20):
    for b in range(20):
        mem[a,b] = algs.value(t,j,x,y,xx[a],yy[b])

fig = plt.figure()
ax = fig.add_subplot(111, projection='3d')
ax.plot_wireframe(X, Y, mem)
print mem
plt.show()
#sim.algs.update()


import Input
import numpy as np

class SIM_ALP:
    def __init__(self,t = 5, T = 40):
        self.reader = Input.Input()
        self.reader.readIn('data/rm_200_4_1.0_4.0.txt')
        self.reader.product()
        self.reader.leg()
        self.reader.construct()
        self.t = t
        self.T = T
        self.set_para()
        
    def set_para(self):

        self.i = self.reader.f
        self.j = self.reader.i
        
        #Sparse P
        #self.P = np.zeros((20000,20000),dtype=np.float)
        
        #construct A
        self.A = {}
        self.c = np.zeros((self.i,1),dtype=np.float)
        self.v = np.zeros((self.j,1),dtype=np.float)

        self.prob = np.zeros((self.t*self.T,self.j), dtype=np.float)
        for t in range(0,self.t * self.T):
            for j in range(0,self.j):
                index = self.reader.prdic[j]
                self.prob[t,j] = self.reader.bdic[(t,index[0],index[1],index[2])]

        #construct A
        for item in self.reader.listA:
        	if item[1]-1 in self.A.keys():
        		self.A[item[1]-1].append(item[0])
        	else:
        		self.A[item[1]-1] = [item[0]]
        #construct c
        for item in range(0,self.i):
            self.c[item] = self.reader.flight[item][2]
            #self.c[item] = 2
        #construct v
        for j in range(0,self.j):
            self.v[j] = self.reader.pval[j]

    def sim(self):
        tmpDemand = np.zeros((self.t*self.T,self.j), dtype=np.float)
        for t in range(0,self.t*self.T):
            p = np.random.uniform()
            for j in range(0,self.j):
                p-= self.prob[t,j]
                if p<=0 :
                    tmpDemand[t,j] += 1
                    break
        self.realDemand = np.zeros((self.t,self.j))
        for t in range(0,self.t):
            for j in range(0,self.j):
                self.realDemand[t,j] = np.sum(tmpDemand[t*self.T:(t+1)*self.T,j])
        return self.realDemand
        


from DR import *
from mabinter import *
import numpy as np
class processer(object):
    def __init__(self,script):
        self.DR = DR(script)
        self.script = script

    def update(self):
        self.DR.update()

    def set_inter(self,inter):
        self.inter = inter

    def test(self,sample):
        C = self.script.C.copy()
        V = self.script.V.copy()
        val = 0
        for t in range(self.script.T):
            product = self.DR.strategy(t, sample[0:t,:])
            
            product = product.reshape(self.script.J)
            
            for j in range(self.script.J):
                product[j] = self.inter(t,j,product[j])
                
            product = np.fmin(product, sample[t,:].T)

            for j in range(self.script.J):
                for l in self.script.A[j]:
                    product[j] = min(product[j],C[l])

                val += product[j] * V[j]
                for l in self.script.A[j]:
                    C[l] -= product[j]

        return val

    def train(self,data):
        self.mab_inter = mabinter(data,self.script)#Todo
        self.inter = self.mab_inter.inter
        self.mab_inter.train(self.test)


import numpy as np
class mabinter(object):
    def __init__(self,data,script):
        self.data = data
        self.script = script

    def inter(self,t,j,v):
        return np.floor(v) + self.v[t,j]

    def train(self,test_fun):
        self.test_fun = test_fun
        self.v = np.ones((self.script.T,self.script.J))
        cur_val = 0.
        for i in range(70):
            new_cur_val = self.bandit(cur_val)
            if new_cur_val == cur_val:
                break
            else:
                cur_val = new_cur_val

    def bandit(self,cur_val):
        T = self.script.T
        J = self.script.J
        
        stage = (T+J)*7
        eps = 0.2
        reward = {}
        pull_num = {}
        for i in range(stage):
            if i == 0:
                arm = (np.random.randint(T), np.random.randint(J))
                self.v[arm[0],arm[1]] = 1 - self.v[arm[0],arm[1]]
                m = np.random.randint(len(self.data)) - 1
                reward[arm] = self.test_fun(self.data[m])
                pull_num[arm] = 1
                self.v[arm[0],arm[1]] = 1 - self.v[arm[0],arm[1]]
                cur_max = reward[arm]
                best = arm
            else:
                if np.random.rand() < 1 - eps:
                    arm = best
                    self.v[arm[0],arm[1]] = 1 - self.v[arm[0],arm[1]]
                    m = np.random.randint(len(self.data)) - 1
                    reward[arm] += self.test_fun(self.data[m])
                    pull_num[arm] += 1
                    self.v[arm[0],arm[1]] = 1 - self.v[arm[0],arm[1]]
                    cur_max = reward[arm] / float(pull_num[arm])
                    for a in reward:
                        if reward[a] / float(pull_num[a]) > cur_max:
                            best = a
                            cur_max = reward[a] / float(pull_num[a])
                else:
                    arm = (np.random.randint(T), np.random.randint(J))
                    while arm == best[0]*T+best[1]:
                        arm = (np.random.randint(T), np.random.randint(J))
                    self.v[arm[0],arm[1]] = 1 - self.v[arm[0],arm[1]]
                    m = np.random.randint(len(self.data)) - 1
                    if arm not in reward.keys():
                        reward[arm] = self.test_fun(self.data[m])
                        pull_num[arm] = 1
                    else:
                        reward[arm] += self.test_fun(self.data[m])
                        pull_num[arm] += 1
                    self.v[arm[0],arm[1]] = 1 - self.v[arm[0],arm[1]]
                    if reward[arm] / float(pull_num[arm]) > cur_max:
                        best = arm
                        cur_max = reward[arm] / float(pull_num[arm])
        if cur_max > cur_val:
            self.v[best[0],best[1]] = 1 - self.v[best[0],best[1]]
            return cur_max
        else:
            return cur_val


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Mon May 15 20:30:41 2017

@author: lin
"""

from SIM import *
from SIM_Resolve_First import *
from piecewise_linear import *
from tri_linear import *
from opt_tri_linear import *

case = SIM_Resolve_First()
#algs = piecewise_linear()
#algs = tri_linear()
algs = opt_tri_linear()
sim = SIM(case,algs)

data = []
for i in range(1000):
    data.append(sim.simulator.sim())
algs.set_train_data(data)

sim.update()
#sim.train_mab()
print sim.avg(1000)


import numpy as np
from algs import *

class piecewise_linear(algs):

    def piecewise(self):
        self.constant = self.script.constant
        self.xi = self.script.xi
        self.div_num = 7
        self.div_axis = {}
        for t in range(self.T):
            for j in range(self.J):
                for k in range(self.div_num):
                    self.div_axis[t,j,k] = self.low_bound[t,j] + (self.up_bound[t,j]-self.low_bound[t,j])/float(self.div_num - 1) * k
                    self.xi[t,j,k] = self.script.add_var(0,self.div_axis[t,j,k])

        for t in range(self.T):
            for j in range(self.J):
                left = {}
                for k in range(self.div_num):
                    left[self.xi[t,j,k]] = 1
                right = {}
                right[self.constant] = 1
                self.script.add_lin_equ(left,right)
                for k in range(self.div_num):
                    self.script.add_lin_greater({self.xi[t,j,k]:1},{self.constant:0})

        for t in range(self.T):
            for j in range(self.J):
                left = {self.xi[t,j]:1}
                right = {}
                for k in range(self.div_num):
                    right[self.xi[t,j,k]] = self.div_axis[t,j,k]
                self.script.add_lin_equ(left,right)

    def value(self,t,j,k,v):
        if k == self.div_num - 1 and v >= self.div_axis[t,j,self.div_num-1] :
            return 1
        elif k < self.div_num-1 and v >= self.div_axis[t,j,k+1]:
            return 0

        if k == 0 and v <= self.div_axis[t,j,0]:
            return 1
        elif k > 0 and v <= self.div_axis[t,j,k-1]:
            return 0

        if v > self.div_axis[t,j,k]:
            return (self.div_axis[t,j,k+1] - v) / (self.div_axis[t,j,k+1] - self.div_axis[t,j,k])
        else:
            return (v-self.div_axis[t,j,k-1]) / (self.div_axis[t,j,k] - self.div_axis[t,j,k-1])


    def lift_value(self,data):
        value = np.zeros((self.script.index,1))
        value[self.constant] = 1
        for t in range(self.T):
            for j in range(self.J):
                value[self.xi[t,j]] = data[t,j]
                for k in range(self.div_num):
                    value[self.xi[t,j,k]] = self.value(t,j,k,data[t,j])
        return value

    def relation(self):
        self.P = {}
        for t in range(self.T):
            for j in range(self.J):
                self.P[t,j] = []
                for k in range(self.div_num):
                    self.P[t,j].append(self.xi[t,j,k])

    def update(self):
        self.piecewise()
        self.relation()
        
    def strategy(self,t,history,X):
        product = np.zeros((self.script.J,1))
        for j in range(self.script.J):
            product[j] = X[t,j,self.xi[t,j,self.div_num-1]].X
                   
        return product


# -*- coding: utf-8 -*-
"""
Created on Wed Aug 10 15:42:43 2016

@author: Zhan
"""

import numpy as np
import scipy as sp
from scipy.special import gamma
from scipy.stats import gamma as Gamma
from scipy.stats import poisson
from scipy.integrate import quad
import Input

class CustomizeDemand:
    def __init__(self,choose, t=10, limt=0, d=7, T = 20):
        self.t = t
        self.limt = min(limt,self.t)
        self.T = T
        self.d = d
        self.lenMon = 10000
        self.lenMonSec = 10000
        self.a = 7
        self.b = 10
        self.r = self.a * self.b
        
        if choose ==0:
            self.reader = self.reductionALPReadIn()
            self.reductionALP(self.reader)
            self.sim = self.produceDemandForrALP
        elif  choose == 1:
            self.resolveDemandFirstCase()
            self.sim = self.produceDemandForFirstCaseInResolve
        elif choose == 2:
            self.resolveDemandSecondCase()
            self.sim = self.produceDemandForSecondCaseInResolve
    
    def avg(self,pt,pj,start,threshold,minus,diff):
        result = 0.0
        for i in range(start,self.lenMon):
            if self.monteCarlo[pt,pj][i] > threshold:
                return result,i
            result += self.monteCarlo[pt,pj][i]-minus
        return result,self.lenMon
    
    def resolveDemandFirstCase(self):   
        self.i = 10
        self.j = 60
        
        #Fare
        self.v = np.zeros((self.j,1))
        for j in range(0,10):
            self.v[2*j] = 300
            self.v[2*j+1] = 80
        for j in range(10,30):
            self.v[2*j] = 500
            self.v[2*j+1] = 100
            
        #Capacity
        self.c = np.zeros((self.i,1))
        for i in range(0,10):
            self.c[i] = 400
        
        #matrix A I * J
        self.A = {}
        for j in range(0,10):
            self.A[2*j] = [j]
            self.A[2*j+1] = [j]
            
        for a in range(0,5):
            for b in range(0,5):
                if a == b:
                    continue
                j = j + 1

                self.A[2*j] = [2*a,2*b+1]
                self.A[2*j+1] = [2*a,2*b+1]
            
        totalLen = 1000.0
        div = int(totalLen / self.t)
        self.cons = np.zeros((self.t,2))
        for t in range(0,self.t):            
            for tt in range(t*div,(t+1)*div):
                self.cons[t,0] +=  0.25 * 1.0/totalLen * (float(tt)/totalLen) ** (6 - 1) * (1- float(tt)/totalLen) ** (2-1) * gamma(8)/gamma(2)/gamma(6)
                self.cons[t,1] +=  0.75 * 1.0/totalLen * (float(tt)/totalLen) ** (2 - 1) * (1- float(tt)/totalLen) ** (6-1) * gamma(8)/gamma(2)/gamma(6)
                
        minf = 0.01
        msup = 0.99
        mxinf = 0.01
        mxsup = 0.99
        self.monteCarlo = {}
        self.lb={}
        self.ub={}
        self.xub = {}
        self.xlb = {}
        for t in range(0,self.t):
            for j in range(0,20):
                simGamma = np.random.gamma(40,size=(self.lenMon))
                if j%2 ==0:
                    self.monteCarlo[t,j] = np.random.poisson(simGamma * self.cons[t,0])
                    self.monteCarlo[t,j].sort()
                else:
                    self.monteCarlo[t,j] = np.random.poisson(simGamma * self.cons[t,1])
                    self.monteCarlo[t,j].sort()
            
            for j in range(20,60):
                simGamma = np.random.gamma(100,size=(self.lenMon))
                if j%2 ==0:
                    self.monteCarlo[t,j] = np.random.poisson(simGamma * self.cons[t,0])
                    self.monteCarlo[t,j].sort()
                else:
                    self.monteCarlo[t,j] = np.random.poisson(simGamma * self.cons[t,1])
                    self.monteCarlo[t,j].sort()
                    
        self.yub = np.zeros((self.t,self.j),dtype=np.float)
        self.ylb = np.zeros((self.t,self.j),dtype=np.float)
        for t in range(1,self.t):
            for j in range(self.j):
                self.yub[t,j] = self.ub[t-1,j] + self.yub[t-1,j]
                self.ylb[t,j] = self.lb[t-1,j] + self.ylb[t-1,j]
        for j in range(self.j):
            self.yub[0,j] = 1
        self.produceMesh()
        tmp = {}
        cu = {}
        self.xi = np.zeros((self.t,self.j,self.a,self.b),dtype=np.float)
        for s in range(self.lenMonSec):
            #simulation
            for t in range(self.t):
                
                simGamma = np.random.gamma(40)
                for j in range(20):
                    if j%2 ==0:
                        tmp[t,j] = np.random.poisson(simGamma * self.cons[t,0])
                    else:
                        tmp[t,j] = np.random.poisson(simGamma * self.cons[t,1])
                
                simGamma = np.random.gamma(100)
                for j in range(20,60):
                    if j%2 ==0:
                        tmp[t,j] = np.random.poisson(simGamma * self.cons[t,0])
                    else:
                        tmp[t,j] = np.random.poisson(simGamma * self.cons[t,1])
            #calculate
            for j in range(self.j):
                cu[j] = 0
            for t in range(self.t):
                for j in range(self.j):
                    result = self.pieceWiseFunctionOnMesh(t,j,tmp[t,j],cu[j]+(t==0))
                    for (a,b,value) in result:
                        self.xi[t,j,a,b] += value
                    cu[j] += tmp[t,j]
        #mean
        for t in range(self.t):
            for j in range(self.j):
                for a in range(self.a):
                    for b in range(self.b):
                        self.xi[t,j,a,b] /= float(self.lenMonSec)
        self.buildWh()
        
    def buildWh(self):
        #W
        #First Block
        self.row = 8*self.t*self.j+2+self.t*self.j*self.r
        self.col = self.t*self.j+self.r*self.t*self.j+1
        row = []
        col = []
        data = []
        for t in range(self.t):
            for j in range(self.j):
                row += [2*(t*self.j+j),2*(t*self.j+j)+1]
                col += [t*self.j+j,t*self.j+j]
                data += [1,-1]
        #Second Block
        rowBase = self.t*self.j*2
        colBase = self.t*self.j
        for t in range(self.t):
            for j in range(self.j):
                for r in range(self.r):
                    row += [rowBase + 2*(t*self.j+j)]
                    col += [colBase + (t*self.j+j)*self.r+r]
                    data += [1]
                for r in range(self.r):
                    row += [rowBase + 2*(t*self.j+j) + 1]
                    col += [colBase + (t*self.j+j)*self.r+r]
                    data += [-1]
        #Last Block
        rowBase = 4*self.t*self.j
        for j in range(self.j):
            row += [rowBase + 4*j,rowBase + 4*j+1,rowBase + 4 * j + 2 , rowBase + 4*j+3 ]
            col += [j,j,self.t*self.j+self.t*self.j*self.r,self.t*self.j+self.t*self.j*self.r]
            data += [1,-1,1,-1]
            for a in range(self.a):
                for b in range(self.b):
                    row += [rowBase + 4*j]
                    col += [self.t*self.j+j*self.r+a*self.b+b]
                    data += [-self.mesh[0,j][0][a]]
            
            for a in range(self.a):
                for b in range(self.b):
                    row += [rowBase + 4*j+1]
                    col += [self.t*self.j+j*self.r+a*self.b+b]
                    data += [self.mesh[0,j][0][a]]
            
            for a in range(self.a):
                for b in range(self.b):
                    row += [rowBase + 4*j+2]
                    col += [self.t*self.j+j*self.r+a*self.b+b]
                    data += [-self.mesh[0,j][1][b]]
            
            for a in range(self.a):
                for b in range(self.b):
                    row += [rowBase + 4*j+3]
                    col += [self.t*self.j+j*self.r+a*self.b+b]
                    data += [self.mesh[0,j][1][b]]
        colBase = self.t*self.j
        for t in range(1,self.t):
            for j in range(self.j):
                
                row += [rowBase + 4*(t*self.j+j),rowBase + 4*(t*self.j+j)+1]
                col += [t*self.j+j,t*self.j+j]                
                data += [1,-1]
                
                for tt in range(t):
                    row += [rowBase + 4*(t*self.j+j)+2]
                    col += [tt*self.j+j]
                    data += [1]
                
                for tt in range(t):
                    row += [rowBase + 4*(t*self.j+j)+3]
                    col += [tt*self.j+j]
                    data += [-1]
                        
                for a in range(self.a):
                    for b in range(self.b):
                        row += [rowBase + 4*(t*self.j+j)]
                        col += [colBase + (t*self.j+j)*self.r+a*self.b+b]
                        data += [-self.mesh[t,j][0][a]]
                
                for a in range(self.a):
                    for b in range(self.b):
                        row += [rowBase + 4*(t*self.j+j)+1]
                        col += [colBase + (t*self.j+j)*self.r+a*self.b+b]
                        data += [self.mesh[t,j][0][a]]
                
                for a in range(self.a):
                    for b in range(self.b):
                        row += [rowBase + 4*(t*self.j+j)+2]
                        col += [colBase + (t*self.j+j)*self.r+a*self.b+b]
                        data += [-self.mesh[t,j][1][b]]
                
                for a in range(self.a):
                    for b in range(self.b):
                        row += [rowBase + 4*(t*self.j+j)+3]
                        col += [colBase + (t*self.j+j)*self.r+a*self.b+b]
                        data += [self.mesh[t,j][1][b]]
            
        row += [self.t*self.j*8,self.t*self.j*8+1]
        col += [self.t*self.j+self.t*self.j*self.r,self.t*self.j+self.t*self.j*self.r]
        data += [1,-1]
        rowBase = self.t*self.j*8+2
        colBase = self.t*self.j
        for t in range(self.t):
            for j in range(self.j):
                for  r in range(self.r):
                    row += [rowBase + (t*self.j+j)*self.r+r]
                    col += [colBase + (t*self.j+j)*self.r+r]
                    data += [1]
        
        self.w = sp.sparse.csc_matrix((data,(row,col)),\
        shape=(self.t*self.j*8+2+self.t*self.j*self.r,self.t*self.j+self.t*self.j*self.r+1))
        
        data = []
        for t in range(self.t):
            for j in range(self.j):
                data += [self.xlb[t,j]]
                data += [-self.xub[t,j]]
        for t in range(self.t):
            for j in range(self.j):
                data += [1,-1]
        for t in range(self.t):
            for j in range(self.j):
                data += [0,0,0,0]
        data += [1,-1]
        data += [0] * (self.t*self.j*self.r)
        self.h = np.array(data)

        
    def tri(self,t,j,x1,y1,x2,y2,x3,y3,x,y):#
        x1 = self.mesh[t,j][0][x1]
        x2 = self.mesh[t,j][0][x2]
        x3 = self.mesh[t,j][0][x3]
        y1 = self.mesh[t,j][1][y1]
        y2 = self.mesh[t,j][1][y2]
        y3 = self.mesh[t,j][1][y3]
        dx1 = x2-x1
        dy1 = y2-y1
        dx2 = x3-x1
        dy2 = y3-y1
        A = np.array([[dx1,dy1],[dx2,dy2]])
        b = np.array([1,1])
        xx = np.linalg.solve(A,b)
        a = xx[0]
        b = xx[1]
        d = -1
        return -(a*(x-x1)+b*(y-y1)+d)
    
    def pieceWiseFunctionOnMesh(self,t,j,x,y):#
        x = max(min(self.ub[t,j],x),self.lb[t,j])
        y = max(min(self.yub[t,j],y),self.ylb[t,j])
        flag = 0
        for a in range(self.a):
            if self.mesh[t,j][0][a] >= x:
                if self.mesh[t,j][0][a] == x:
                    flag = 1
                break
        for b in range(self.b):
            if self.mesh[t,j][1][b] >= y :
                if self.mesh[t,j][1][b] == y:
                    if flag == 1:
                        flag = 3
                    else:
                        flag = 2
                break
        if flag == 0 :
            dy = y-self.mesh[t,j][1][b-1]
            dx = x-self.mesh[t,j][0][a]
            ddy = self.mesh[t,j][1][b] - self.mesh[t,j][1][b-1]
            ddx = self.mesh[t,j][0][a-1] - self.mesh[t,j][0][a]
            if dx*ddy - ddx*dy>0:
                result = [(a-1,b,self.tri(t,j,a-1,b,a,b,a,b-1,x,y)), (a,b,self.tri(t,j,a,b,a-1,b,a,b-1,x,y)), \
                (a,b-1,self.tri(t,j,a,b-1,a,b,a-1,b,x,y))]
            else:
                result = [(a-1,b-1,self.tri(t,j,a-1,b-1,a-1,b,a,b-1,x,y)), (a-1,b,self.tri(t,j,a-1,b,a,b-1,a-1,b-1,x,y)), \
                (a,b-1,self.tri(t,j,a,b-1,a-1,b-1,a-1,b,x,y))]
        elif flag == 1:
                result = [(a,b-1,1-(y-self.mesh[t,j][1][b-1])/float(self.mesh[t,j][1][b]-self.mesh[t,j][1][b-1])), \
                        (a,b,(y-self.mesh[t,j][1][b-1])/float(self.mesh[t,j][1][b]-self.mesh[t,j][1][b-1]))]
        elif flag == 2:
                result = [(a-1,b,1-(x-self.mesh[t,j][0][a-1])/float(self.mesh[t,j][0][a]-self.mesh[t,j][0][a-1])), \
                        (a,b,(x-self.mesh[t,j][0][a-1])/float(self.mesh[t,j][0][a]-self.mesh[t,j][0][a-1]))]
        else:
            result = [(a,b,1)]
        return result
    
    def produceMesh(self):
        self.mesh = {}
        for t in range(self.t):
            for j in range(self.j):
                self.mesh[t,j] = [np.linspace(self.lb[t,j],self.ub[t,j],self.a),\
                np.linspace(self.ylb[t,j],self.yub[t,j],self.b)]
    
    def produceDemandForFirstCaseInResolve(self):        
        self.realDemand = []
        for t in range(0,self.t):
            for j in range(0,10):
                g = np.random.gamma(40)
                self.realDemand += [np.random.poisson(g * self.cons[t][0])]
                self.realDemand += [np.random.poisson(g * self.cons[t][1])]
            for j in range(10,30):
                g = np.random.gamma(100)
                self.realDemand += [np.random.poisson(g * self.cons[t][0])]
                self.realDemand += [np.random.poisson(g * self.cons[t][1])]
        return self.realDemand
                
    def resolveDemandSecondCase(self):
        self.i = 10
        self.j = 60
        
        #Fare
        self.v = np.zeros((self.j,1))
        for j in range(0,10):
            self.v[2*j] = 300
            self.v[2*j+1] = 80
        for j in range(10,22):
            self.v[2*j] = 500
            self.v[2*j+1] = 100
        for j in range(22,30):
            self.v[2*j] = 700
            self.v[2*j+1] = 200
            
        #Capacity
        self.c = np.zeros((self.i,1))
        for i in range(0,10):
            self.c[i] = 400
        self.c[4] = self.c[5] = 1000
        
        #matrix A I * J
        self.A = np.zeros((self.i,self.j),dtype=np.int8)
        self.refJ = {}
        for j in range(0,10):
            self.A[j,2*j] = 1
            self.A[j,2*j+1] = 1
            self.refJ[2*j] = [j]
            self.refJ[2*j+1] = [j]
        self.pushTwoLeg(0,1,0,3,1,2,0)
        self.pushTwoLeg(0,5,0,4,5,1,1)
        self.pushTwoLeg(1,5,2,4,5,3,2)
        self.pushTwoLeg(2,4,6,5,4,7,3)
        self.pushTwoLeg(2,3,6,8,9,7,4)
        self.pushTwoLeg(3,4,9,5,4,8,5)
        self.pushThreeLeg(0,2,0,4,7,6,5,1,6)
        self.pushThreeLeg(0,3,0,4,8,9,5,1,7)
        self.pushThreeLeg(1,2,2,4,7,6,5,3,8)
        self.pushThreeLeg(1,3,2,4,8,9,5,3,9)
        
        totalLen = 1000.0
        div = int(totalLen / self.t)
        self.cons = np.zeros((self.t,2))
        for t in range(0,self.t):            
            for tt in range(t*div,(t+1)*div):
                self.cons[t,0] +=  0.25 * 1.0/totalLen * (float(tt)/totalLen) ** (6 - 1) * (1- float(tt)/totalLen) ** (2-1) * gamma(8)/gamma(2)/gamma(6)
                self.cons[t,1] +=  0.75 * 1.0/totalLen * (float(tt)/totalLen) ** (2 - 1) * (1- float(tt)/totalLen) ** (6-1) * gamma(8)/gamma(2)/gamma(6)
                
        self.monteCarlo = {}
        #Segmentation 
        self.h = []
        self.seg = {}
        minsup = 0.01
        mininf = 0.01
        for t in range(0,self.t):
            for j in range(0,20):
                simGamma = np.random.gamma(60,size=(self.lenMon))
                if j%2 ==0:
                    self.monteCarlo[t,j] = np.random.poisson(simGamma * self.cons[t,0])
                    self.monteCarlo[t,j].sort()
                    b = self.monteCarlo[t,j][int(np.ceil((1-minsup)*self.lenMon-1))]
                    a = self.monteCarlo[t,j][int(np.floor(mininf*self.lenMon))]
                else:
                    self.monteCarlo[t,j] = np.random.poisson(simGamma * self.cons[t,1])
                    self.monteCarlo[t,j].sort()
                    b = self.monteCarlo[t,j][int(np.ceil((1-minsup)*self.lenMon-1))]
                    a = self.monteCarlo[t,j][int(np.floor(mininf*self.lenMon))]
                new = []
                for d in range(0,self.d):
                    new += [float(b-a)/self.d*d+a]
                new += [b]
                self.seg[t,j] = new
                
            for j in range(20,44):
                simGamma = np.random.gamma(150,size=(self.lenMon))
                if j%2 ==0:
                    self.monteCarlo[t,j] = np.random.poisson(simGamma * self.cons[t,0])
                    self.monteCarlo[t,j].sort()
                    b = self.monteCarlo[t,j][int(np.ceil((1-minsup)*self.lenMon-1))]
                    a = self.monteCarlo[t,j][int(np.floor(mininf*self.lenMon))]
                else:
                    self.monteCarlo[t,j] = np.random.poisson(simGamma * self.cons[t,1])
                    self.monteCarlo[t,j].sort()
                    b = self.monteCarlo[t,j][int(np.ceil((1-minsup)*self.lenMon-1))]
                    a = self.monteCarlo[t,j][int(np.floor(mininf*self.lenMon))]
                new = []
                for d in range(0,self.d):
                    new += [float(b-a)/self.d*d+a]
                new += [b]
                self.seg[t,j] = new
                
            for j in range(44,60):
                simGamma = np.random.gamma(100,size=(self.lenMon))
                if j%2 ==0:
                    self.monteCarlo[t,j] = np.random.poisson(simGamma * self.cons[t,0])
                    self.monteCarlo[t,j].sort()
                    b = self.monteCarlo[t,j][int(np.ceil((1-minsup)*self.lenMon-1))]
                    a = self.monteCarlo[t,j][int(np.floor(mininf*self.lenMon))]
                else:
                    self.monteCarlo[t,j] = np.random.poisson(simGamma * self.cons[t,1])
                    self.monteCarlo[t,j].sort()
                    b = self.monteCarlo[t,j][int(np.ceil((1-minsup)*self.lenMon-1))]
                    a = self.monteCarlo[t,j][int(np.floor(mininf*self.lenMon))]
                new = []
                for d in range(0,self.d):
                    new += [float(b-a)/self.d*d+a]
                new += [b]
                self.seg[t,j] = new
                
        #Expectation of arrival process
        self.xi = np.zeros((self.t*self.j*self.d+1,1),dtype=np.float)
        self.xi[0] = 1
        for t in range(0,self.t):
            for j in range(0,30):
                left = 0
                leftSec = 0
                for d in range(0,self.d):
                    low = self.seg[t,2*j][d]
                    up = self.seg[t,2*j][d+1]
                    if d == 0:
                        low = 0
                    self.xi[1+(t*self.j+2*j)*self.d+d],left =  self.avg(t,2*j,left,up,low)
                    self.xi[1+(t*self.j+2*j)*self.d+d] += (up - low)* (self.lenMon - left)
                    self.xi[1+(t*self.j+2*j)*self.d+d] /= self.lenMon
                    
                    low = self.seg[t,2*j+1][d]
                    up = self.seg[t,2*j+1][d+1]
                    if d == 0:
                        low = 0
                    self.xi[1+(t*self.j+2*j+1)*self.d+d],leftSec =  self.avg(t,2*j+1,leftSec,up,low)
                    self.xi[1+(t*self.j+2*j+1)*self.d+d] += (up - low)* (self.lenMon - leftSec)
                    self.xi[1+(t*self.j+2*j+1)*self.d+d] /= self.lenMon 
        #Up Bound And Low Bound For Demand
        #print self.xi
        #print self.h
    
    def pushTwoLeg(self,s,t,a,b,c,d,id):
        self.A[a,20+4*id] = self.A[b,20+4*id] = 1
        self.A[a,20+4*id+1] = self.A[b,20+4*id+1] = 1
        self.A[c,20+4*id+2] = self.A[d,20+4*id+2] = 1
        self.A[c,20+4*id+3] = self.A[d,20+4*id+3] = 1
        self.refJ[20+4*id] = [a,b]
        self.refJ[20+4*id+1] = [a,b]
        self.refJ[20+4*id+2] = [c,d]
        self.refJ[20+4*id+3] = [c,d]
    
    def pushThreeLeg(self,s,t,a,b,c,d,e,f,id):
        self.A[a,20+4*id] = self.A[b,20+4*id] = self.A[c,20+4*id] = 1
        self.A[a,20+4*id+1] = self.A[b,20+4*id+1] = self.A[c,20+4*id+1] = 1
        self.A[d,20+4*id+2] = self.A[e,20+4*id+2] = self.A[f,20+4*id+2] = 1
        self.A[d,20+4*id+3] = self.A[e,20+4*id+3] = self.A[f,20+4*id+3] = 1
        self.refJ[20+4*id] = [a,b,c]
        self.refJ[20+4*id+1] = [a,b,c]
        self.refJ[20+4*id+2] = [d,e,f]
        self.refJ[20+4*id+3] = [d,e,f]

    def produceDemandForSecondCaseInResolve(self):    
        self.realDemand = []
        for t in range(0,self.t):
            for j in range(0,10):
                g = np.random.gamma(60)
                self.realDemand += [np.random.poisson(g * self.cons[t][0])]
                self.realDemand += [np.random.poisson(g * self.cons[t][1])]
            for j in range(10,22):
                g = np.random.gamma(150)
                self.realDemand += [np.random.poisson(g * self.cons[t][0])]
                self.realDemand += [np.random.poisson(g * self.cons[t][1])]
            for j in range(22,30):
                g = np.random.gamma(100)
                self.realDemand += [np.random.poisson(g * self.cons[t][0])]
                self.realDemand += [np.random.poisson(g * self.cons[t][1])]
        return self.realDemand

    def reductionALPReadIn(self):
        reader = Input.Input()
        reader.readIn('data/rm_200_4_1.0_4.0.txt')
        reader.product()
        reader.leg()
        reader.construct()
        return reader
        
    def reductionALP(self,relrALP):

        self.i = relrALP.f
        self.j = relrALP.i
        
        #Sparse P
        #self.P = np.zeros((20000,20000),dtype=np.float)
        
        #construct A
        self.A = np.zeros((self.i,self.j),dtype=np.float)
        self.prob = np.zeros((self.t*self.T*self.j+1,1),dtype=np.float)
        self.c = np.zeros((self.i,1),dtype=np.float)
        #Sparse W
        #self.W = np.zeros((2*(self.t*self.j+1),self.t*self.j+1),dtype=np.float)
        self.v = np.zeros((self.j,1),dtype=np.float)
        self.rALP = relrALP
        #look resources I required by J
        self.refJ = relrALP.dicAforJ
        #construct A
        for item in self.rALP.listA:
            #Hint Item[1] is from 1 to ...
            #print item
            self.A[item[0],item[1]-1] = 1
        #construct c
        for item in range(0,self.i):
            self.c[item] = self.rALP.flight[item][2]
            #self.c[item] = 2
        #construct Exi
        self.prob = np.zeros((self.t*self.T,self.j), dtype=np.float)
        for t in range(0,self.t * self.T):
            for j in range(0,self.j):
                index = self.rALP.prdic[j]
                self.prob[t,j] = self.rALP.bdic[(t,index[0],index[1],index[2])]
        self.monteCarlo = {}
        minf = 0.01
        msup = 0.99
        mxsup = 0.9
        mxinf = 0.1
        self.xub = np.zeros((self.t,self.j),dtype=np.float)
        self.xlb = np.zeros((self.t,self.j),dtype=np.float)
        self.ub = np.zeros((self.t,self.j),dtype=np.float)
        self.lb = np.zeros((self.t,self.j),dtype=np.float)
        for t in range(0,self.t):
            for j in range(0,self.j):
                self.monteCarlo[t,j] = []
                for k in range(0,self.lenMon):
                    self.monteCarlo[t,j].append(np.sum(np.random.uniform(size=self.T)<self.prob[t*self.T:(t+1)*self.T,j]))
                self.monteCarlo[t,j].sort()
                self.xub[t,j] = self.monteCarlo[t,j][int(np.ceil(mxsup*self.lenMon-1))]
                self.xlb[t,j] = self.monteCarlo[t,j][int(np.floor(mxinf*self.lenMon))]
                self.ub[t,j] = self.monteCarlo[t,j][int(np.ceil(msup*self.lenMon-1))]
                #print b
                self.lb[t,j] = self.monteCarlo[t,j][int(np.floor(minf*self.lenMon))]
                       
        self.yub = np.zeros((self.t,self.j),dtype=np.float)
        self.ylb = np.zeros((self.t,self.j),dtype=np.float)
        for t in range(1,self.t):
            for j in range(self.j):
                self.yub[t,j] = self.ub[t-1,j] + self.yub[t-1,j]
                self.ylb[t,j] = self.lb[t-1,j] + self.ylb[t-1,j]
        for j in range(self.j):
            self.yub[0,j] = 1
        self.produceMesh()
        tmp = {}
        cu = {}
        self.xi = np.zeros((self.t,self.j,self.a,self.b),dtype=np.float)
        for s in range(self.lenMonSec):
            #simulation
            for t in range(self.t):
                for j in range(self.j):
                    tmp[t,j]=np.sum(np.random.uniform(size=self.T)<self.prob[t*self.T:(t+1)*self.T,j])
            #calculate
            for j in range(self.j):
                cu[j] = 0
            for t in range(self.t):
                for j in range(self.j):
                    result = self.pieceWiseFunctionOnMesh(t,j,tmp[t,j],cu[j]+(t==0))
                    for (a,b,value) in result:
                        self.xi[t,j,a,b] += value
                    cu[j] += tmp[t,j]
        #mean
        for t in range(self.t):
            for j in range(self.j):
                for a in range(self.a):
                    for b in range(self.b):
                        self.xi[t,j,a,b] /= float(self.lenMonSec)        
        self.buildWh()
        #print self.xi
        #construct v
        for j in range(0,self.j):
            self.v[j] = self.rALP.pval[j]
        #for t in range(0,self.t):
            #for j in range(0,self.j):
                #self.xi[1+t*self.j+j] *= 100
        #print sum(self.xi)

    def produceDemandForrALP(self):
        tmpDemand = np.zeros((self.t*self.T,self.j), dtype=np.float)
        for t in range(0,self.t*self.T):
            p = np.random.uniform()
            for j in range(0,self.j):
                p-= self.prob[t,j]
                if p<=0 :
                    tmpDemand[t,j] += 1
                    break
        self.realDemand = []
        for t in range(0,self.t):
            for j in range(0,self.j):
                self.realDemand += [np.sum(tmpDemand[t*self.T:(t+1)*self.T,j])]
        return self.realDemand
                
    def cal(self,t,j,minp):
        s = 0
        for T in range(0,self.T):
            index = self.rALP.prdic[j]
            p = self.rALP.bdic[(t*self.T+T,index[0],index[1],index[2])]
            i = 0
            q = p
            tmp = 1
            while(q>minp):
                i = i+1
                tmp = tmp*(i+1)
                q = (p**(i+1))/tmp
            s += i
        return s
        


from distutils.core import setup

setup(
    name='pyldr',
    version='0.1.1',
    modules=['script'],
    author='Zhan Lin',
    author_email='dy403164418@gmail.com',
    packages=['pyldr'],
    license='MIT License',
    description='Python Package for Linear Decision Rule and Generalized Decision Rule',
    long_description=open('README.txt').read(),
    install_requires=[
        "numpy >= 1.11.3",
        "scipy >= 0.18.1",
    ],
)



import algs
import numpy as np
class linear(algs):
	def __init__(self):
		algs.__init__(self):

	def piecewise(self):
		self.constant = self.script.constant
		self.xi = self.script.xi
		
		for t in range(self.T):
			for j in range(self.J):
				for t_pre in range(t):
					for j_pre in range(j):
						self.xi[t,j,t_pre,j_pre] = self.script.add_var()

	def lift_value(self,data):
		value = np.zeros((self.script.index,1))
		value[self.constant] = 1
		for t in range(self.T):
			for j in range(self.J):
				value[self.xi[t,j]] = data[t,j]
				for t_pre in range(t):
					for j_pre in range(j):
						value[self.xi[t,j,t_pre,j_pre]] = data[t,j]
		return value

	def get_relation(self):
		return self.P

	def relation(self):
		self.P = {}
		for t in range(self.T):
			for j in range(self.J):
				self.P[t,j] = []
				for t_pre in range(t):
					for j_pre in range(j):
						self.P[t,j].append(self.xi[t,j,t_pre,j_pre])

	def update(self):
		self.piecewise()
		self.relation()


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Sun Feb 26 19:47:42 2017

@author: root
"""

import NETWORK
import numpy as np

from scipy.special import gamma
from scipy.stats import gamma as Gamma
from scipy.stats import poisson

net = NETWORK()

for i in range(20):
    net.addProfit(i,500)
    
for i in range(20,30):
    net.addProfit(i,300)

for i in range(30,50):
    net.addProfit(i,100)

for i in range(50,60):
    net.addProfit(i,80)

for i in range(10):
    net.addRes(i,400)
    
for i in range(10):
    net.addUsage(20+i,[i])
    net.addUsage(50+i,[i])

net.addUsage(0,[0,1])
net.addUsage(1,[0,2])
net.addUsage(2,[0,3])
net.addUsage(3,[0,4])
net.addUsage(4,[1,0])
net.addUsage(5,[1,2])
net.addUsage(6,[1,3])
net.addUsage(7,[1,4])
net.addUsage(8,[2,0])
net.addUsage(9,[2,1])
net.addUsage(10,[2,3])
net.addUsage(11,[2,4])
net.addUsage(12,[3,0])
net.addUsage(13,[3,1])
net.addUsage(14,[3,2])
net.addUsage(15,[3,4])
net.addUsage(16,[4,0])
net.addUsage(17,[4,1])
net.addUsage(18,[4,2])
net.addUsage(19,[4,3])


net.addUsage(30,[0,1])
net.addUsage(31,[0,2])
net.addUsage(32,[0,3])
net.addUsage(33,[0,4])
net.addUsage(34,[1,0])
net.addUsage(35,[1,2])
net.addUsage(36,[1,3])
net.addUsage(37,[1,4])
net.addUsage(38,[2,0])
net.addUsage(39,[2,1])
net.addUsage(40,[2,3])
net.addUsage(41,[2,4])
net.addUsage(42,[3,0])
net.addUsage(43,[3,1])
net.addUsage(44,[3,2])
net.addUsage(45,[3,4])
net.addUsage(46,[4,0])
net.addUsage(47,[4,1])
net.addUsage(48,[4,2])
net.addUsage(49,[4,3])

p = 10
Origin = 1000.0
div = int(Origin / p)
cons = np.zeros((p,2))
for t in range(0,p):            
    for tt in range(t*div,(t+1)*div):
        cons[t,0] +=  0.25 * 1.0/Origin * (float(tt)/Origin) ** (6 - 1) * (1- float(tt)/Origin) ** (2-1) * gamma(8)/gamma(2)/gamma(6)
        cons[t,1] +=  0.75 * 1.0/Origin * (float(tt)/Origin) ** (2 - 1) * (1- float(tt)/Origin) ** (6-1) * gamma(8)/gamma(2)/gamma(6)
        
monteSize = 1000
monteCarlo = np.zeros((t,60))
real = []
for k in range(monteSize):
    for t in range(0,t):
        simGamma = np.random.gamma(100,size=(1))
        for j in range(20):
            monteCarlo[t,j] = np.random.poisson(simGamma * cons[t,0])
        for j in range(30,50):
            monteCarlo[t,j] = np.random.poisson(simGamma * cons[t,1])
        simGamma = np.random.gamma(40,size=(1))
        for j in range(20,30):
            monteCarlo[t,j] = np.random.poisson(simGamma * cons[t,0])
        for j in range(50,60):
            monteCarlo[t,j] = np.random.poisson(simGamma * cons[t,1])
    net.addDemand(monteCarlo)

net.update()